--------------------------------------------------------------------------------
--
--  file:    LuaMetalSpots-v4.lua
--  brief:   Lua metal spots for *A mods. Credits to Niobium for metal finder. 
--
--   Changelog:
--	 v4.0
--	 * Rewrote from scratch using Niobium metal finder available globally in BA.
--
--   v3.3
--   * fixed issues with archive loading
--   v3.2
--   * fixed drawing on sloped terrain
--   * randomness and multiple textures is broken
--
--   v3.1
--   * fixed depth testing issue
--
--   v3
--   * changed to Niobium metal finder with mass metal detection.
--
--   v2
--   * added display lists
--   * added loading metal maps from file
--
--
--  author:  Jahziah Wagner
--   
--  Copyright (C) 2010.
--  Licensed under the terms of the LGPL
--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
function widget:GetInfo()
  return {
    name      = "Lua Metal Patches V4",
    desc      = "Requires Niobium's metal finder.",
    author    = "Cheesecan/Halcyon",
    date      = "03/01/2012",
    license   = "LGPL v2",
    layer     = 5,
    enabled   = true  --  loaded by default?
  }
end

-- Variables
local displayList 		= 0
local metalSpotWidth    = 64
local metalSpotHeight   = 64
-- End variables

function widget:Initialize()
	if not WG.metalSpots then
		Spring.Echo("<Lua Metal Patches> This widget requires the 'Metalspot Finder' widget to run.")
		widgetHandler:RemoveWidget(self)
	end
	
	displayList = gl.CreateList(drawPatches)
end

function drawPatches()
   local mSpots = WG.metalSpots

    gl.PolygonOffset(-25, -2)
    gl.Culling(GL.BACK)
    gl.DepthTest(true)
	gl.Texture("maps/metal_high.png" )
	gl.Color(1, 1, 1) -- fix color from other widgets
	
	for i = 1, #mSpots do
	  --gl.PushMatrix()
	  --gl.LoadIdentity();
	  --gl.Translate( mSpots[i].x, mSpots[i].y, mSpots[i].z )
      --gl.Rotate( math.rad(math.random(0, 180)), 0, 0, 1 )
	  gl.DrawGroundQuad( mSpots[i].x - metalSpotWidth/2, mSpots[i].z - metalSpotHeight/2, mSpots[i].x + metalSpotWidth/2, mSpots[i].z + metalSpotHeight/2, false, 0, 0, 1, 1)
	  --gl.PopMatrix()
	end
   
    gl.Texture(false)
    gl.DepthTest(false)
    gl.Culling(false)
    gl.PolygonOffset(false)
end

function widget:DrawWorldPreUnit()
   gl.CallList(displayList)
end